//-------------------------------------------------------------------------------------------------
// TheApplication.h (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#ifndef _include_TheApplication_
#define _include_TheApplication_


//-------------------------------------------------------------------------------------------------
// Easy-access macro
//-------------------------------------------------------------------------------------------------
#define APPLICATION (TheApplication::Singleton())


//-------------------------------------------------------------------------------------------------
// Included header files
//-------------------------------------------------------------------------------------------------
#include "engineCommon.h"


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//		TheApplication
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
class TheApplication
{
private:
	/// Static members
	static TheApplication* _theApplication;
	static bool _isConstructed;
	static bool _isInitialized;
	static std::string _appName; // Application name
//	static std::string _baseDir; // Base directory to which all paths are relative

	/// Private members
	std::string				_cmdLine;		// Contains a copy of the command line with which the program was run
	HINSTANCE				_hInstance;		// A copy of <hInstance> as it was passed into WinMain()
	bool					_isExiting;		// if true, the application is in the process of shutting down
	bool					_isShutdownRequested;	// if true, the application has received a shutdown request
	unsigned int			_frameNumber;
	double					_secondsSinceLastFrame;
	double					_timeFrameBegan;
	float					_fpsAsymptotic;
	bool					_hasFocus;

	/// Private methods
	void _Shutdown();
	void _ProcessCommandLine();
	void _UpdateFrameInfo();

	/// Private constructors
	TheApplication( const HINSTANCE& hInstance, const LPSTR& lpCmdLine );

	//---------------------------------------------------------------------------
	// Begin public class interface
	//---------------------------------------------------------------------------
public:
	enum
	{
		EXIT_MESSAGE_NORMAL = 0,
		EXIT_MESSAGE_ERROR_GENERIC = 1,
	};

	/// Construction, initialization, and singleton methods
	~TheApplication();

	static TheApplication* CreateSingleton( const HINSTANCE& hInstance, const LPSTR& lpCmdLine );
	static TheApplication* Singleton();
	static void DestroySingleton();

	void Init();
	void RequestShutdown();
	void CheckForShutdownRequest();

	/// Static methods
	static inline const std::string& GetName() { return _appName; }
	static inline void SetName( const std::string& appName ) { _appName = appName; }
//	static inline const String& GetBaseDir() { return _baseDir; }

	/// Public accessors / mutators
	inline const bool IsExiting() const { return _isExiting; }
	inline const bool HasFocus() const { return _hasFocus; }
	inline const HINSTANCE GetHInstance() const { return _hInstance; }
	inline unsigned int GetFrameNumber() const { return _frameNumber; }
	inline double GetFrameSeconds() const { return _secondsSinceLastFrame; }
	inline double GetTimeFrameBegan() const { return _timeFrameBegan; }

	/// Public methods
	void OnQuitMessage();
	void OnAppGainFocus();
	void OnAppLoseFocus();
	void RunFrame();
	inline float GetFpsAsymptotic() const { return _fpsAsymptotic; }
};
	//---------------------------------------------------------------------------
	// End public class interface
	//---------------------------------------------------------------------------


#endif // _include_TheApplication_



