//-------------------------------------------------------------------------------------------------
// TheConsole.h (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#ifndef _include_TheConsole_
#define _include_TheConsole_


//-------------------------------------------------------------------------------------------------
// Easy-access macro
//-------------------------------------------------------------------------------------------------
#define CONSOLE (TheConsole::Singleton())


//-------------------------------------------------------------------------------------------------
// Forward self-declaration
//-------------------------------------------------------------------------------------------------
//class TheConsole;


//-------------------------------------------------------------------------------------------------
// Included header files
//-------------------------------------------------------------------------------------------------
#include "engineCommon.h"


//-------------------------------------------------------------------------------------------------
// Console channels
//-------------------------------------------------------------------------------------------------
enum
{
	CHAN_DEFAULT			= BIT( 0 ),
	CHAN_LOW				= BIT( 1 ),
	CHAN_HIGH				= BIT( 2 ),

	CHAN_APPLICATION		= BIT( 3 ),
	CHAN_GAME				= BIT( 4 ),
	CHAN_INPUT				= BIT( 5 ),
	CHAN_MESSAGE			= BIT( 6 ),
	CHAN_NETWORK			= BIT( 7 ),
	CHAN_RENDERER			= BIT( 8 ),
	CHAN_VIDEO				= BIT( 9 ),
	CHAN_SOUND				= BIT( 10 )
};


///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		TheConsole
//
///////////////////////////////////////////////////////////////////////////////////////////////////
class TheConsole
{
private:

/*	struct ConsoleLine
	{
		bitflags_t		channels;
		std::string		text;
	};
*/
	/// Static members
	static TheConsole* _theConsole;
	static bool _isConstructed;
	static bool _isInitialized;

	/// Private members
	bitflags_t _activeChannels;
	std::vector<std::string> _consoleText;
	int _maxLines;
	int _numLines;
	int _lastLine;
	int _lineCount;

	/// Private methods
	void _WipeConsole();
	void _PrintChars( const char* text );
	void _PrintLine( const char* text, int textLength );
	void _DebugDumpConsole();
	void _DebugDumpConsoleLine( int lineNum, int printLineNum );
	int _GetLineIndexForPrintLineNum( int printLineNum );

	/// Private constructors
	TheConsole(); // default constructor

public:
	/// Construction, initialization, and singleton methods
	~TheConsole(); // destructor

	static TheConsole* CreateSingleton();
	static TheConsole* Singleton();
	static void DestroySingleton();

	void Init();
	void Shutdown();

	/// Public methods
	inline void EnableChannels( bitflags_t channelFlags ) { _activeChannels |= channelFlags; }
	inline void DisableChannels( bitflags_t channelFlags ) { _activeChannels &= ~channelFlags; }
	inline void SetActiveChannels( bitflags_t channelFlags ) { _activeChannels = channelFlags; }

	void Printf( const char* format, ... );
	void Printf( bitflags_t channels, const char* format, ... );
	const std::string& GetTextForPrintLineNumber( int printLineNum );
	int GetHighestPrintLineNumber() const;
	int GetLowestPrintLineNumber() const;
};



#endif // _include_TheConsole_
