//-------------------------------------------------------------------------------------------------
// TheGame.cpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.h"
#include "TheGame.h"


//-------------------------------------------------------------------------------------------------
// Globals
//-------------------------------------------------------------------------------------------------
TheGame* g_game = NULL;


//-------------------------------------------------------------------------------------------------
// Destructor()
//-------------------------------------------------------------------------------------------------
TheGame::~TheGame()
{
}


//-------------------------------------------------------------------------------------------------
// Constructor
//-------------------------------------------------------------------------------------------------
TheGame::TheGame()
{
}


//-------------------------------------------------------------------------------------------------
// PreInit
//-------------------------------------------------------------------------------------------------
void TheGame::PreInit()
{
	/// Assign application name
//	APPLICATION->SetName( "Test Application" );

	/// Choose a video mode
//	VIDEO->SetDefaultVidModeProperties( 1280, 960, 32, 85, TheVideoSystem::VIDMODE_FLAG_FULLSCREEN );
	VIDEO->SetDefaultVidModeProperties( 640, 480, 32, 85, 0 );

	/// Set the mouse in hide & recenter mode
//	INPUT->UseWindowsMouseCursor( false );
}


//-------------------------------------------------------------------------------------------------
// PostInit
//-------------------------------------------------------------------------------------------------
void TheGame::PostInit()
{
	RENDERER->ShowFpsToggle();
}


//-------------------------------------------------------------------------------------------------
// RunFrame
//-------------------------------------------------------------------------------------------------
void TheGame::RunFrame()
{
	/// Get the size and center of the current video mode
	Vector2 screenSize( (float) VIDEO->GetCurrentVidModeWidth(), (float) VIDEO->GetCurrentVidModeHeight() );
	Vector2 screenCenter( screenSize * 0.5f );

	/// Set up a 2d drawing coordinate system with 0.0 at the bottom-left
	RENDERER->SetCoordinateSystem( Vector2::ZERO, screenSize );

	/// Print a test message in the center of the screen
	RENDERER->XYPrintf( screenCenter.x, screenCenter.y, Rgba::RED, "Press ESC to quit" );

	/// Get the screen (client rect) mouse cursor position
	int mouseX, mouseY;
	INPUT->GetMousePosClientAbsolute( mouseX, mouseY );

	/// Flip the Y axis (client rect has 0.0 at top-left; we're bottom-left)
	Vector2 mouse( (float) mouseX, (float) (VIDEO->GetCurrentVidModeHeight() - mouseY) );

	/// Draw a something random... like a shaded vector from mouse position to a rotating position
	double timeNow = APPLICATION->GetTimeFrameBegan(); // get current time
	double radians = 3.0 * timeNow; // rotate around a circle with time
	Vector2 pos( radians ); // construct a unit vector from radian angle
	pos *= screenCenter.y; // scale rotation radius to be half-screen-height
	pos += screenCenter; // move rotation fulcrum to screen center
	RENDERER->DrawVector( mouse, pos, Rgba::RED, Rgba::GREEN ); // draw a shaded vector from mouse to pos
}


//-------------------------------------------------------------------------------------------------
// OnKeyDown
//-------------------------------------------------------------------------------------------------
void TheGame::OnKeyDown( const byte_t key )
{
	if( key == VK_ESCAPE )
	{
		APPLICATION->RequestShutdown();
	}
	else if( key == VK_F11 )
	{
		RENDERER->ShowFpsToggle();
	}
	else if( key == VK_TILDE )
	{
		RENDERER->ShowConsoleToggle();
	}
}


//-------------------------------------------------------------------------------------------------
// OnKeyUp
//-------------------------------------------------------------------------------------------------
void TheGame::OnKeyUp( const byte_t key )
{
}


